<?php

namespace Drupal\strings_i18n_json_export\Service;

use Drupal\Component\Serialization\Json;
use Drupal\Core\Language\LanguageManagerInterface;
use Drupal\Core\StreamWrapper\PublicStream;

/**
 * StringsI18nJsonExport Service Class.
 */
class StringsI18nJsonExport {

  /**
   * The language manager.
   *
   * @var \Drupal\Core\Language\LanguageManagerInterface
   */
  protected $languageManager;

  /**
   * Construct.
   */
  public function __construct(LanguageManagerInterface $language_manager) {
    $this->languageManager = $language_manager;
  }

  /**
   * Method to export all Json Files.
   */
  public function exportAllJsonFiles($log = NULL) {

    $languages = \Drupal::languageManager()->getLanguages();

    foreach ($languages as $lid => $language) {
      self::generateJsonFile($lid, $log);
    }
  }

  /**
   * Method to generate Json File.
   */
  public function generateJsonFile(string $lid, $log = NULL) {

    $config = \Drupal::config('string_translation_ui.settings');

    $contexts = [];
    if (!empty($config->get('contexts_used'))) {
      $contexts = Json::decode($config->get('contexts_used'));
      sort($contexts);
    }

    $json = $this->getTranslationsJson($lid, $contexts);

    if (empty($json)) {

      \Drupal::logger('strings_i18n_json_export')->error(t('Error on export file for language: @language', [
        '@language' => $lid,
      ]));

      return FALSE;
    }

    return $this->exportJsonFile($json, $lid, $log);
  }

  /**
   * Method to export JSON file.
   */
  protected function exportJsonFile($json, $lid, $log = NULL) {

    // Get public folder.
    $publicPath = PublicStream::basePath();
    $path = \Drupal::service('file_system')->realpath($publicPath);
    $path .= '/lang';

    // Create directory.
    $directory = file_prepare_directory($path, FILE_CREATE_DIRECTORY);

    // Check if directory was created.
    if (empty($directory)) {

      \Drupal::logger('strings_i18n_json_export')->error(t('Error creating folder: @folder', [
        '@folder' => $path,
      ]));

      return FALSE;
    }

    $fileName = $lid . '.json';
    $fullPath = $path . '/' . $fileName;

    // Delete old file.
    if (file_exists($fullPath)) {
      unlink($fullPath);
    }

    // Save the new file.
    $saved = file_put_contents($fullPath, $json);

    if (!$saved) {
      \Drupal::logger('strings_i18n_json_export')->error(t('Error on export file'));
      return FALSE;
    }

    if ($log) {
      \Drupal::logger('strings_i18n_json_export')->info(t('File i18n for language: @language has been exported. Path: @path', [
        '@language' => $lid,
        '@path' => $fullPath,
      ]));
    }

    // Return file path.
    return $fullPath;
  }

  /**
   * Method to get JSON.
   */
  public function getTranslationsJson(string $lid, array $contexts = []) {

    $data = [];
    $strings = '';

    foreach ($contexts as $context) {

      $data[$context] = [];

      $strings = \Drupal::service('locale.storage')->getTranslations(['context' => $context, 'language' => $lid]);

      $source = '';
      $translation = '';

      foreach ($strings as $string) {

        $source = $string->source;
        $translation = $string->translation;

        if (empty($translation)) {
          $translation = $source;
        }

        $data[$context][$source] = $translation;
      }
    }

    $json = Json::encode($data);

    return $json;
  }

}
